// Copyright 2004-2008 John Eblen
// Department of Computer Science
// University of Tennessee
//
// This file is part of Paraclique.            
//
// Paraclique is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by   
// the Free Software Foundation, either version 3 of the License, or   
// (at your option) any later version.   
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of   
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   
// GNU General Public License for more details.   
//
// You should have received a copy of the GNU General Public License   
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// July 17, 2006
// Base class for Clique Test functional classes plus some derived classes
// that implement specific tests.

// Very simple base class - derived classes need only implement the
// procedure call ( () operator) with the appropriate parameters.

#ifndef _CLIQUE_TEST_H
#define _CLIQUE_TEST_H 1

#include <map>
#include "Graph.h"
#include "color.h"
using namespace std;

class Clique_Test
{
  protected:
  const bool subgraph_needed;

  public:
  Clique_Test(bool b) :subgraph_needed(b) {}
  virtual bool operator() (const Graph::Vertices &, int, int node = -1)
                                                                  const = 0;
  bool need_subgraph() const {return subgraph_needed;}
  virtual ~Clique_Test() {}
};

class Size_Test : public Clique_Test
{
  public:
  Size_Test() :Clique_Test(0) {}

  bool operator() (const Graph::Vertices &v, int min_size, int node) const
  {
    if (node != -1) return v.degree(node) < min_size - 1;
    return v.size() < min_size;
  }
};

class Edge_Test : public Clique_Test
{
  public:
  Edge_Test() :Clique_Test(1) {}

  bool operator() (const Graph::Vertices &v, int min_size, int node) const;
};

class Color_Test : public Clique_Test
{
  public:
  Color_Test() :Clique_Test(1) {}

  bool operator() (const Graph::Vertices &v, int min_size, int node) const
  {
    if (node != -1)
    {
      cerr << "Error - color test called with specific node." << endl;
      exit(EXIT_FAILURE);
    }

    return color_subgraph(v, 0, 0) < min_size;
  }
};

#endif
