// Copyright 2004-2008 John Eblen
// Department of Computer Science
// University of Tennessee
//
// This file is part of Paraclique.            
//
// Paraclique is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by   
// the Free Software Foundation, either version 3 of the License, or   
// (at your option) any later version.   
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of   
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   
// GNU General Public License for more details.   
//
// You should have received a copy of the GNU General Public License   
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "Degree_Preprocessor.h"

int Degree_Preprocessor(Graph::Vertices &mygraph, Graph::Vertices &clique,
                                                               int *ldnode)
{
  int num_added = 0;
  int deg, size;
  int lowest_degree = mygraph.size();
  Graph::Vertices::Vex_ptr vp(mygraph);

  if (ldnode != 0) *ldnode = -1;
  for (; !vp.end(); ++vp)
  {
    deg = mygraph.degree(*vp);
    size = mygraph.size();

    // n-1 test:  Vertex connected to all others must be in clique
    if (deg == size-1)
    {
      mygraph.remove(*vp);
      clique.insert(*vp);
      ++num_added;
    }

    // n-2 test:  Vertex connected to all but one must be in clique - delete
    // the single non-neighbor.
    else if (deg == size-2)
    {
      mygraph.remove_nnbrs(*vp);
      mygraph.remove(*vp);
      clique.insert(*vp);
      ++num_added;
    }

    else if (deg < lowest_degree)
    {
      if (ldnode != 0) *ldnode = *vp;
      lowest_degree = deg;
    }
  }

  return num_added;
}
