// Copyright 2004-2008 John Eblen
// Department of Computer Science
// University of Tennessee
//
// This file is part of Paraclique.            
//
// Paraclique is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by   
// the Free Software Foundation, either version 3 of the License, or   
// (at your option) any later version.   
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of   
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   
// GNU General Public License for more details.   
//
// You should have received a copy of the GNU General Public License   
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// July 28, 2006
// Base class for heuristic algorithms for maximum clique plus some specific,
// derived algorithms.  This class encapsulates an algorithm for quickly
// computing a lower bound on the maximum clique size.  These algorithms
// often are the first step of a larger algorithm to solve the exact maximum
// clique problem.  For example, the lower bound can serve as a good minimum
// clique size parameter for preprocessing.  Effective algorithms will vary
// for different types of graphs, but, theoretically, there are no guarantees.

#ifndef _MC_HEURISTIC_H
#define _MC_HEURISTIC_H 1

#include "Graph.h"
using namespace std;

class MC_Heuristic
{
  public:
  // First three operators simply provide the user with a flexible
  // interface.  The heuristic is actually implemented in the fourth
  // operation.
  int operator() (Graph &g) const
  {
    Graph::Vertices subgraph(g);
    Graph::Vertices clique(g);
    subgraph.fill();
    clique.clear();
    return operator() (subgraph, clique);
  }

  int operator() (Graph &g, Graph::Vertices &clique) const
  {
    Graph::Vertices subgraph(g);
    subgraph.fill();
    clique.clear();
    return operator() (subgraph, clique);
  }
  
  int operator() (Graph::Vertices &subgraph) const
  {
    Graph *base_graph = subgraph.graph();
    Graph::Vertices clique(*base_graph);
    clique.clear();
    return operator() (subgraph, clique);
  }

  virtual int operator() (Graph::Vertices &, Graph::Vertices &) const = 0;
  virtual ~MC_Heuristic() {}
};

class Greedy_Clique : public MC_Heuristic
{
  public:
  using MC_Heuristic::operator();
  int operator() (Graph::Vertices &, Graph::Vertices &) const;
};

#endif
