// Copyright 2004-2008 John Eblen
// Department of Computer Science
// University of Tennessee
//
// This file is part of Paraclique.            
//
// Paraclique is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by   
// the Free Software Foundation, either version 3 of the License, or   
// (at your option) any later version.   
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of   
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   
// GNU General Public License for more details.   
//
// You should have received a copy of the GNU General Public License   
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "color.h"

int color_subgraph(const Graph::Vertices &sg, vector<int> *node2color,
                                              vector<int> *sorted_nodes)
{
  int i;
  int num_nodes_left = sg.size();
  int current_color = 0;
  Graph::Vertices color_nbrs(sg);

  // Warning:  Makes certain assumptions about static nature of the graph.
  // See graph library for details.
  vector<int> nodes;
  sg.sorted_nodes_vector(nodes);

  // Each iteration applies a single color.  Continue until all nodes are
  // colored.
  while(num_nodes_left > 0)
  {
    color_nbrs.clear();

    // Greedily color nodes, skipping those colored previously or neighbors of
    // nodes with the current color.
    for (i=nodes.size()-1; i >= 0; --i)
    {
      if (nodes[i] == -1) continue;
      if (color_nbrs.in_set(nodes[i])) continue;

      if (node2color) (*node2color)[nodes[i]] = current_color;
      if (sorted_nodes) (*sorted_nodes)[num_nodes_left-1] = nodes[i];

      color_nbrs.insert_nbrs(nodes[i]);
      --num_nodes_left;
      nodes[i] = -1;
    }

    ++current_color;
  }

  return current_color;
}
