// Copyright 2004-2008 John Eblen
// Department of Computer Science
// University of Tennessee
//
// This file is part of Paraclique.            
//
// Paraclique is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by   
// the Free Software Foundation, either version 3 of the License, or   
// (at your option) any later version.   
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of   
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   
// GNU General Public License for more details.   
//
// You should have received a copy of the GNU General Public License   
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "paracl.h"

void build_paracl(Graph &g, Graph::Vertices &v, int glom)
{
  int i;
  vector<int> nodes;
  Graph::Vertices all_nodes(g);
  all_nodes.fill();
  all_nodes.node_array(nodes);
  Graph::Vertices new_members(g);

  do
  {
    new_members.clear();
    for (i=0; i<g.size(); i++)
    {
      if (!v.in_set(nodes[i]))
      {
        if ((v.size() - v.degree(nodes[i])) <= glom) new_members.insert(nodes[i]);
      }
    }
    v.insert(new_members);
    cerr << "Added " << new_members.size() << " for a paraclique of size " << v.size() << endl;
  } while (new_members.size() > 0);
}

void build_acc_paracl(Graph &g, Graph::Vertices &v, int agg_factor)
{
  int i;
  int glom = 1;
  vector<int> nodes;
  Graph::Vertices all_nodes(g);
  all_nodes.fill();
  all_nodes.node_array(nodes);
  Graph::Vertices new_members(g);

  do
  {
    new_members.clear();
    for (i=0; i<g.size(); i++)
    {
      if (!v.in_set(nodes[i]))
      {
        if ((v.size() - v.degree(nodes[i])) <= glom) new_members.insert(nodes[i]);
      }
    }
    v.insert(new_members);
    ++glom;
    cerr << "Added " << new_members.size() << " for a paraclique of size " << v.size() << endl;
  } while (agg_factor * glom <= v.size());
}
